<?php

namespace App\Jobs;


use App\Exceptions\CampaignPausedException;
use App\Library\Exception\QuotaExceeded;
use App\Library\QuotaManager;
use App\Models\Automation;
use App\Models\SendingServer;
use App\Models\User;
use DateTime;
use Exception;
use Illuminate\Bus\Batchable;
use Throwable;

class SendAutomationMessage extends Base
{
    use Batchable;

    protected Automation    $automation;
    protected               $contacts;
    protected SendingServer $server;
    protected User          $user;
    protected bool          $stopOnError = false;
    protected array         $priceOption;

    /**
     * Create a new job instance.
     */
    public function __construct($automation, $contact, $server, $user, $priceOption)
    {
        $this->automation  = $automation;
        $this->contacts    = $contact;
        $this->server      = $server;
        $this->user        = $user;
        $this->priceOption = $priceOption;
    }

    /**
     * @throws Exception
     */
    public function setStopOnError($value): void
    {
        if ( ! is_bool($value)) {
            throw new Exception('Parameter passed to setStopOnError must be bool');
        }

        $this->stopOnError = $value;
    }

    /**
     * Determine the time at which the job should timeout.
     *
     * @return DateTime
     */
    public function retryUntil(): DateTime
    {
        return now()->addHours(12);
    }


    /**
     * @throws QuotaExceeded|Exception
     */
    public function handle(): void
    {
        if ($this->batch() && $this->batch()->cancelled()) {
            return;
        }

        $this->send();

    }

    /**
     * Execute the job.
     *
     * @throws Exception
     */
    public function send($exceptionCallback = null): void
    {

        $subscription = $this->user->customer->getCurrentSubscription();

        try {

            if ($this->user->sms_unit != '-1' && $this->user->sms_unit == 0) {
                throw new CampaignPausedException(sprintf("Automation `%s` (%s) halted, customer exceeds sms balance", $this->automation->name, $this->automation->uid));
            }

            QuotaManager::with($subscription, 'send')->enforce();
            QuotaManager::with($this->server, 'send')->enforce();

            $sent  = $this->automation->send($this->contacts, $this->priceOption, $this->server);
            $msgId = $sent->id;

            $this->automation->track_message($sent->toArray(), $this->contacts, $this->server, $msgId);
        } catch (QuotaExceeded $ex) {
            if ( ! is_null($exceptionCallback)) {
                $exceptionCallback($ex);
            }
            $this->release(60);
        } catch (CampaignPausedException $ex) {
            if ( ! is_null($exceptionCallback)) {
                $exceptionCallback($ex);
            }
            $this->automation->pause($ex->getMessage());

        } catch (Throwable $ex) {
            if ( ! is_null($exceptionCallback)) {
                $exceptionCallback($ex);
            }
            $message = sprintf("Error sending to [%s]. Error: %s", $this->contacts, $ex->getMessage());
            if ($this->stopOnError) {
                throw new Exception($message);
            } else {
                if ( ! isset($msgId)) {
                    // Just in case there is an exception before the execution of "list($message, $msgId) = $this->campaign->prepareEmail..."
                    // then $msgID is not available
                    $msgId = null;
                }

                $this->automation->track_message(['status' => 'failed', 'error' => $ex->getMessage()], $this->contacts, $this->server, $msgId);
            }
        }

    }
}
