<?php

namespace App\Jobs;

use App\Exceptions\CampaignPausedException;
use App\Library\Exception\QuotaExceeded;
use App\Library\QuotaManager;
use App\Models\Campaigns;
use App\Models\Contacts;
use App\Models\SendingServer;
use DateTime;
use Exception;
use Illuminate\Bus\Batchable;
use Throwable;

class SendMessage extends Base
{
    use Batchable;


    public int $timeout = 600;

    protected Contacts      $contact;
    protected Campaigns     $campaign;
    protected SendingServer $server;
    protected array         $priceOption;
    protected bool          $stopOnError = false;

    /**
     * Create a new job instance.
     */
    public function __construct($campaign, Contacts $contact, $server, $priceOption)
    {
        $this->campaign    = $campaign;
        $this->contact     = $contact;
        $this->server      = $server;
        $this->priceOption = $priceOption;
    }

    /**
     * @throws Exception
     */
    public function setStopOnError($value): void
    {
        if ( ! is_bool($value)) {
            throw new Exception('Parameter passed to setStopOnError must be bool');
        }

        $this->stopOnError = $value;
    }


    /**
     * Determine the time at which the job should timeout.
     *
     * @return DateTime
     */
    public function retryUntil(): DateTime
    {
        return now()->addHours(12);
    }


    /**
     * @throws QuotaExceeded
     */
    public function handle(): void
    {
        if ($this->batch() && $this->batch()->cancelled()) {
            return;
        }

        $this->send();

    }

    /**
     * @throws QuotaExceeded
     * @throws Exception
     */
    public function send($exceptionCallback = null): void
    {
        $subscription = $this->campaign->user->customer->getCurrentSubscription();

        try {

            if ($this->campaign->user->sms_unit != '-1' && $this->campaign->user->sms_unit == 0) {
                throw new CampaignPausedException(sprintf("Campaign `%s` (%s) halted, customer exceeds sms balance", $this->campaign->campaign_name, $this->campaign->uid));
            }

            QuotaManager::with($subscription, 'send')->enforce();
            QuotaManager::with($this->server, 'send')->enforce();

            $sent  = $this->campaign->send($this->contact, $this->priceOption, $this->server);
            $msgId = $sent->id;

            $this->campaign->track_message($sent->toArray(), $this->contact, $this->server, $msgId);
        } catch (QuotaExceeded $ex) {
            if ( ! is_null($exceptionCallback)) {
                $exceptionCallback($ex);
            }
            $this->release(60);
        } catch (CampaignPausedException $ex) {
            if ( ! is_null($exceptionCallback)) {
                $exceptionCallback($ex);
            }
            $this->campaign->pause($ex->getMessage());

        } catch (Throwable $ex) {
            if ( ! is_null($exceptionCallback)) {
                $exceptionCallback($ex);
            }
            $message = sprintf("Error sending to [%s]. Error: %s", $this->contact, $ex->getMessage());
            if ($this->stopOnError) {
                throw new Exception($message);
            } else {
                if ( ! isset($msgId)) {
                    // Just in case there is an exception before the execution of "list($message, $msgId) = $this->campaign->prepareEmail..."
                    // then $msgID is not available
                    $msgId = null;
                }

                $this->campaign->track_message(['status' => 'failed', 'error' => $ex->getMessage()], $this->contact, $this->server, $msgId);
            }
        }

    }
}
